<?php

require_once dirname( __FILE__ ) . '/libs/wedevs-updater.php';

/**
 * The Updater Class
 */
class WeForms_Pro_Updater extends WeDevs_License_Update {

    function __construct( $plan ) {

        parent::__construct( WEFORMS_PRO_FILE, 'weForms Pro', 'weforms-pro', $plan, WEFORMS_PRO_VERSION );

        $this->base_plugin_key = 'weforms_pro';
        $this->api_endpoint    = 'http://api.wedevs.com/';
        $this->option          = 'weforms_pro_license';
        $this->license_status  = 'weforms_pro_license_status';

        if ( ! $this->is_local_server() ) {
            add_action( 'wp_ajax_weforms_pro_get_license', array( $this, 'weforms_license_status' ) );

            add_action( 'weforms-admin-menu', array( $this, 'admin_menu' ), 10, 2 );
            add_filter( 'weforms_vue_routes', array( $this, 'vue_routes' ) );
        }
    }

    /**
     * Add admin menu
     *
     * @param  string $hook
     * @param  string $capability
     *
     * @return void
     */
    public function admin_menu( $hook, $capability ) {
        global $submenu;

        if ( current_user_can( 'manage_options' ) ) {
            $submenu['weforms'][] = array( __( 'License', 'weforms-pro' ), 'manage_options', 'admin.php?page=weforms#/license' );
        }
    }

    /**
     * Add license routes into the router
     *
     * @param  array $routes
     *
     * @return array
     */
    public function vue_routes( $routes ) {
        $routes[] = array(
            'path'      => '/license',
            'name'      => 'license',
            'component' => 'License'
        );

        return $routes;
    }

    /**
     * License status checking
     *
     * @return void
     */
    public function weforms_license_status() {
        $data = array(
            'license' => $this->get_license_key(),
            'status'  => $this->get_license_status(),
            'message' => ''
        );

        if ( isset( $data['status']->update ) ) {
            $update = strtotime( $data['status']->update );
            $expired = false;

            if ( time() > $update ) {
                $string  = __( 'has been expired %s ago', 'weforms-pro' );
                $expired = true;
            } else {
                $string = __( 'will expire in %s', 'weforms-pro' );
            }

            $data['message'] = sprintf( __( 'Your license %s (%s).', 'weforms-pro' ), sprintf( $string, human_time_diff( $update, time() ) ), date( 'F j, Y', $update ) );
        }

        wp_send_json_success( $data );
    }

    /**
     * Prompts the user to add license key if it's not already filled out
     *
     * @return void
     */
    function license_enter_notice() {
        if ( $key = $this->get_license_key() ) {
            return;
        }
        ?>
        <div class="error">
            <p><?php printf( __( 'Please <a href="%s">enter</a> your <strong>%s</strong> plugin license key to get regular update and support.', 'weforms-pro' ), admin_url( 'admin.php?page=weforms#/license' ), $this->name ); ?></p>
        </div>
        <?php
    }
}
