<?php

/**
 * Country Field Class
 */
class WeForms_Form_Field_Country extends WeForms_Field_Contract {

    function __construct() {
        $this->name       = __( 'Country List', 'weforms' );
        $this->input_type = 'country_list_field';
        $this->icon       = 'globe';
    }

    /**
     * Render the text field
     *
     * @param  array  $field_settings
     * @param  integer  $form_id
     *
     * @return void
     */
    public function render( $field_settings, $form_id ) {
        $country_list             = $field_settings['country_list'];
        $list_visibility_option   = $country_list['country_list_visibility_opt_name'];
        $country_select_hide_list = isset( $country_list['country_select_hide_list'] ) && is_array( $country_list['country_select_hide_list'] ) ? $country_list['country_select_hide_list'] : array();
        $country_select_show_list = isset( $country_list['country_select_show_list'] ) && is_array( $country_list['country_select_show_list'] ) ? $country_list['country_select_show_list'] : array();
        $value                    = isset( $country_list['name'] ) ? $country_list['name'] : '';
        ?>
        <li <?php $this->print_list_attributes( $field_settings ); ?>>
            <?php $this->print_label( $field_settings ); ?>

            <div class="wpuf-fields <?php echo ' wpuf_'.$field_settings['name'].'_'.$form_id; ?>">
                <select name="<?php echo $field_settings['name']; ?>"></select>

                <?php $this->help_text( $field_settings ); ?>
            </div>

            <script>
                var field_name             = '<?php echo $field_settings['name'];?>';
                var countries              = <?php echo weforms_pro_get_countries( 'json' ); ?>;
                var banned_countries       = JSON.parse('<?php echo json_encode($country_select_hide_list); ?>');
                var allowed_countries      = JSON.parse('<?php echo json_encode($country_select_show_list); ?>');
                var list_visibility_option = '<?php echo $list_visibility_option; ?>';
                var sel_country            = '<?php echo !empty( $value ) ? $value : '' ; ?>';
                var option_string          = '<option value="">Select Country</option>';

                if ( list_visibility_option == 'hide' ) {

                    for ( country in countries ) {
                        if ( jQuery.inArray(countries[country].code,banned_countries) != -1 ) {
                            continue;
                        }

                        option_string = option_string + '<option value="'+ countries[country].code +'" ' + ( sel_country == countries[country].code ? 'selected':'' ) + ' >'+ countries[country].name +'</option>';
                    }

                } else if ( list_visibility_option == 'show' ) {
                    for (country in countries) {
                        if ( jQuery.inArray(countries[country].code,allowed_countries) != -1 ) {
                            option_string = option_string + '<option value="'+ countries[country].code +'" ' + ( sel_country == countries[country].code ? 'selected':'' ) + ' >'+ countries[country].name +'</option>';
                        }
                    }
                } else {
                    for (country in countries) {
                        option_string = option_string + '<option value="'+ countries[country].code +'" ' + ( sel_country == countries[country].code ? 'selected':'' ) + ' >'+ countries[country].name +'</option>';
                    }
                }

                jQuery('select[name="'+ field_name +'"]').html(option_string);
            </script>

        </li>
        <?php
    }

    /**
     * Get field options setting
     *
     * @return array
     */
    public function get_options_settings() {
        $default_options      = $this->get_default_option_settings( true, array( 'width' ) );

        $settings = array(
            array(
                'name'          => 'country_list',
                'title'         => '',
                'type'          => 'country-list',
                'section'       => 'advanced',
                'priority'      => 22,
                'help_text'     => '',
            )
        );

        return array_merge( $default_options, $settings );
    }

    /**
     * Get the field props
     *
     * @return array
     */
    public function get_field_props() {
        $defaults = $this->default_attributes();
        $props    = array(
            'country_list'  => array(
                'name'                              => '',
                'country_list_visibility_opt_name'  => 'all', // all, hide, show
                'country_select_show_list'          => array(),
                'country_select_hide_list'          => array()
            )
        );

        return array_merge( $defaults, $props );
    }
}